package application;

import debug.*;
import timer.*;
import link.*;
import network.*;

import java.io.*;
import java.util.*;

public class project3
{
   public static void main (String[] args)
   {
      project3 runProject = new project3 ();
      ParseArgs (args);
      runProject.BuildNetworkConfig ();
   }

   public static long SEED = System.currentTimeMillis ();

   public static String CONFIG_FILENAME = "project2.cfg";

   private static void ParseArgs (String[] args)
   {
      for (int i=0; i<args.length; ++i)
         {
         int eqidx = args[i].indexOf ('=');
         if (eqidx == -1)
            continue;

         String prefix = args[i].substring (0, eqidx);
         String value = args[i].substring (eqidx + 1);

         if (prefix.equals ("seed"))
            SEED = ParseLong (value, 0);

         else if (prefix.equals ("config"))
            {
            CONFIG_FILENAME = value;
            }

         else if (prefix.equals ("debug"))
            Debug.SetEnabled (value);
         }

      for (int i=0; i<args.length; ++i)
         {
         if (args[i].equals ("-flag"))
            {
            }
         }
   }

   private static long ParseLong (String s, long def)
   {
      try
         {
         def = Long.parseLong (s);
         }
      catch (NumberFormatException e)
         {
         }

      return def;
   }

   private static double ParseDouble (String s, double def)
   {
      try 
         {
         def = Double.valueOf (s).doubleValue ();
         }
      catch (NumberFormatException e)
         {
         }
      catch (NullPointerException e)
         {
         }

      return def;
   }

   private Hashtable routers;
   private Hashtable links;
   private Hashtable networks;
   private Hashtable edges;
   private Hashtable senders;
   private Hashtable receivers;
   private Hashtable sendApps;
   private Hashtable receiveApps;

   private void BuildNetworkConfig ()
   {
      routers = new Hashtable ();
      links = new Hashtable ();
      networks = new Hashtable ();
      edges = new Hashtable ();
      senders = new Hashtable ();
      receivers = new Hashtable ();
      sendApps = new Hashtable ();
      receiveApps = new Hashtable ();

      Timer.global.RegisterThread (Thread.currentThread ());
      synchronized (Timer.global)
         {
         try
            {
            FileInputStream file = new FileInputStream (CONFIG_FILENAME);
            StreamTokenizer input = new StreamTokenizer (file);
            input.commentChar ('#');

            for (;;)
               {
               int ttype = input.nextToken ();
               if (ttype == StreamTokenizer.TT_EOF)
                  {
                  Timer.global.UnregisterThread (Thread.currentThread ());
                  return;
                  }

               if (ttype != StreamTokenizer.TT_WORD)
                  throw new Project3InputParseError (input.lineno ());

               if (input.sval.equals ("Router"))
                  {
                  // Expect name and queue len
                  if (input.nextToken () != '\"')
                     throw new Project3InputParseError (input.lineno ());
                  String name = input.sval;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new Project3InputParseError (input.lineno ());
                  int queueLen = (int) input.nval;

                  if (!routers.containsKey (name))
                     routers.put (name, new Router (name, queueLen));
                  }
               else if (input.sval.equals ("Link"))
                  {
                  Link link = null;

                  // Expect name, class, end, port, 
                  //         mtu, bandwidth est, latency est
                  //   OR
                  // Expect name, class, end1, end2, echohost,
                  //         mtu, bandwidth est, latency est

                  if (input.nextToken () != '\"')
                     throw new Project3InputParseError (input.lineno ());
                  String name = input.sval;

                  if (input.nextToken () != StreamTokenizer.TT_WORD)
                     throw new Project3InputParseError (input.lineno ());
                  String kind = input.sval;

                  if (input.nextToken () != '\"')
                     throw new Project3InputParseError (input.lineno ());
                  String end1 = input.sval;
                  if (!routers.containsKey (end1))
                     throw new Project3InputParseError (input.lineno ());

                  int port = 0;
                  String end2 = "";
                  String echohost = "";

                  if (kind.equals ("UDPLink"))
                     {
                     if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                        throw new Project3InputParseError (input.lineno ());
                     port = (int) input.nval;
                     }
                  else if (kind.equals ("EchoLink") || 
                           kind.equals ("LocalEchoLink"))
                     {
                     if (input.nextToken () != '\"')
                        throw new Project3InputParseError (input.lineno ());
                     end2 = input.sval;
                     if (!routers.containsKey (end2))
                        throw new Project3InputParseError (input.lineno ());

                     if (input.nextToken () != StreamTokenizer.TT_WORD)
                        throw new Project3InputParseError (input.lineno ());
                     echohost = input.sval;
                     }

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new Project3InputParseError (input.lineno ());
                  int mtu = (int) input.nval;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new Project3InputParseError (input.lineno ());
                  double bandwidthEst = input.nval;

                  if (input.nextToken () != StreamTokenizer.TT_NUMBER)
                     throw new Project3InputParseError (input.lineno ());
                  double latencyEst = input.nval;

                  if (kind.equals ("UDPLink"))
                     {
                     link = new UDPLink (name, mtu, port, bandwidthEst, latencyEst);

                     if (!links.containsKey (name))
                        links.put (name, link);

                     Node endNode1 = (Node) routers.get (end1);
                     link.AddNode (endNode1);
                     endNode1.AddLink (link);
                     }
                  else if (kind.equals ("EchoLink") || 
                           kind.equals ("LocalEchoLink"))
                     {
                     if(kind.equals ("EchoLink")) { 
                       link = new EchoLink (name, mtu, echohost, bandwidthEst, latencyEst);
                     } else {
                       link = new LocalEchoLink (name, mtu, echohost, bandwidthEst, latencyEst);
                     }


                     if (!links.containsKey (name))
                        links.put (name, link);

                     Node endNode1 = (Node) routers.get (end1);
                     Node endNode2 = (Node) routers.get (end2);

                     link.AddNode (endNode1);
                     link.AddNode (endNode2);

                     endNode1.AddLink (link);
                     endNode2.AddLink (link);
                     }
                  else
                     throw new Project3InputParseError (input.lineno ());

                  }
               }
            }
         catch (IOException e)
            {
            System.err.println ("ERROR: could not open/read config file.");
            System.exit (1);
            }
        }
   }   
}

class Project3InputParseError extends RuntimeException
{
   public Project3InputParseError (int lineno)
   {
      super ("Parse error at input line: " + lineno);
   }
}

